from typing import Generic, Optional, TypeVar

from .transformer import Transformer
from .result import Result
from .parser import Parser


P = TypeVar("P")
class FutureTransform(Transformer[P]):
    actual: Optional[Transformer[P]]
    def __init__(self) -> None:
        self.actual = None
    
    def define(self, actual: Transformer[P]):
        if self.actual is not None:
            raise Exception("Redefinition of parser.")
        self.actual = actual
    
    def parse(self, stream: str, at_index: int) -> Result[P]:
        if self.actual is None:
            raise Exception("Using forwarded definition of parser without defining first.")
        return self.actual.parse(stream, at_index)


P = TypeVar("P")
class Declare(Generic[P]):
    parser: Parser[P]
    transform: FutureTransform[P]

    def __init__(self) -> None:
        transform = FutureTransform[P]()
        self.parser = Parser(transform)
        self.transform = transform

    def p(self):
        return self.parser

    def define(self, parser: Parser[P]):
        self.transform.define(parser.inner)
