from dataclasses import dataclass
from typing import TypeVar

from ..result import Result, Success, Failure
from ..transformer import Transformer


L = TypeVar("L")
R = TypeVar("R")
@dataclass
class AndTransform(Transformer[tuple[L, R]]):
    def __init__(self, left: Transformer[L], right: Transformer[R]):
        self.left = left
        self.right = right

    def parse(self, stream: str, at_index: int) -> Result[tuple[L, R]]:
        result_left = self.left.parse(stream, at_index)
        if isinstance(result_left, Failure):
            return result_left
        result_right = self.right.parse(stream, result_left.next_index)
        if isinstance(result_right, Failure):
            return result_right
        return Success((result_left.value, result_right.value), result_right.next_index)
