from dataclasses import dataclass

from ..result import Result, Success, Failure
from ..transformer import Transformer


@dataclass
class JustTransform(Transformer[str]):
    def __init__(self, word: str):
        self.word = word
        self.word_len = len(self.word)

    def parse(self, stream: str, at_index: int) -> Result[str]:
        end_index = at_index + self.word_len
        prefix = stream[at_index:end_index]
        if prefix == self.word:
            return Success(self.word, end_index)
        return Failure(at_index, set([self.word]))

