from dataclasses import dataclass
from typing import TypeVar

from ..result import Result, Success, Failure
from ..transformer import Transformer


T = TypeVar("T")
@dataclass
class ListTransform(Transformer[list[T]]):
    def __init__(self, item: Transformer[T]):
        self.item = item

    def parse(self, stream: str, at_index: int) -> Result[list[T]]:
        values = list[T]()
        while True:
            result = self.item.parse(stream, at_index)
            if isinstance(result, Failure):
                break
            values.append(result.value)
            if result.next_index == at_index:
                raise Exception("Parsing empty patterns repeatedly.")
            at_index = result.next_index
        return Success(values, at_index)
