from dataclasses import dataclass
from typing import Callable, TypeVar

from ..result import Result, Success, Failure
from ..transformer import Transformer


I = TypeVar("I")
O = TypeVar("O")
@dataclass
class MapTransform(Transformer[O]):
    def __init__(self, value: Transformer[I], transform: Callable[[I], O]):
        self.value = value
        self.transform = transform

    def parse(self, stream: str, at_index: int) -> Result[O]:
        result = self.value.parse(stream, at_index)
        if isinstance(result, Failure):
            return result
        transformed = self.transform(result.value)
        return Success(transformed, result.next_index)
