from dataclasses import dataclass
from typing import Optional, TypeVar

from ..result import Result, Success
from ..transformer import Transformer


T = TypeVar("T")
@dataclass
class OptionTransform(Transformer[Optional[T]]):
    def __init__(self, value: Transformer[T]):
        self.value = value
    
    def parse(self, stream: str, at_index: int) -> Result[Optional[T]]:
        result = self.value.parse(stream, at_index)
        if isinstance(result, Success):
            return Success(result.value, result.next_index)
        return Success(None, at_index)
