from dataclasses import dataclass
from typing import TypeVar, Union

from ..result import Result, Success, Failure
from ..transformer import Transformer


L = TypeVar("L")
R = TypeVar("R")
@dataclass
class OrTransform(Transformer[Union[L, R]]):
    def __init__(self, left: Transformer[L], right: Transformer[R]):
        self.left = left
        self.right = right

    def parse(self, stream: str, at_index: int) -> Result[Union[L, R]]:
        result_left = self.left.parse(stream, at_index)
        if isinstance(result_left, Success):
            return Success(result_left.value, result_left.next_index)
        result_right = self.right.parse(stream, at_index)
        if isinstance(result_right, Success):
            return Success(result_right.value, result_right.next_index)
        return Failure(at_index, result_left.expected.union(result_right.expected))
