from dataclasses import dataclass
import re

from ..result import Result, Success, Failure
from ..transformer import Transformer


@dataclass
class RegexTransform(Transformer[str]):
    def __init__(self, pattern: str):
        self.pattern = pattern

    def parse(self, stream: str, at_index: int) -> Result[str]:
        do_match = re.match(self.pattern, stream[at_index:])
        if do_match is None:
            return Failure(at_index, set(f"<matching '{self.pattern}'>"))
        else:
            match = do_match[0]
            return Success(match, at_index + len(match))
