from dataclasses import dataclass
from typing import Optional, TypeVar

from ..result import Result
from ..transformer import Transformer

from .map import MapTransform
from .option import OptionTransform
from .and_ import AndTransform
from .list import ListTransform


T = TypeVar("T")
S = TypeVar("S")
@dataclass
class SepListTransform(Transformer[list[T]]):
    def __init__(self, items: Transformer[T], sep: Transformer[S]):
        self.items = items
        self.sep = sep
        first_transformer = OptionTransform(items)
        rest_transformer = ListTransform(AndTransform(sep, items))
        total_transformer = AndTransform(first_transformer, rest_transformer)
        self.actual = MapTransform(total_transformer,SepListTransform.mapping)

    def parse(self, stream: str, at_index: int) -> Result[list[T]]:
        return self.actual.parse(stream, at_index)

    @staticmethod
    def mapping(value: tuple[Optional[T], list[tuple[S, T]]]):
        items = list[T]()
        if value[0] is not None:
            items.append(value[0])
        for (_sep, item) in value[1]:
            items.append(item)
        return items
